<?php
namespace Teknisa\Model\Teknisa\Repositories;

class OmConfigMedidaVariavel extends \Doctrine\ORM\EntityRepository {

    public function findVariableMeasurementSettings($conditions, $nrProdutoId){
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT CMV.ID, CMV.IDATIVO, CMV.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV,
		                CMV.PHP_SCRIPT_COUNT, CMV.PHP_SCRIPT_LOG, CMV.NRPRODUTO_ID,
		                CMV.QTD_DIAS_AVISO, CMV.QTD_DIAS_REAPLICAR,
		                OMP.ID AS NRPRODUTO_ID_REL, OMP.NMPRODUTO AS NRPRODUTO_ID_REL_NAME
                   FROM OM_CONFIG_MEDIDA_VARIAVEL CMV
                   JOIN FCT_TIPOMEDIDASERV TMS ON TMS.CDTPMEDIDASERV = CMV.CDTPMEDIDASERV
              LEFT JOIN OM_PRODUTO OMP ON OMP.ID = CMV.NRPRODUTO_ID_REL
                  WHERE NRPRODUTO_ID = :NRPRODUTO_ID )
		 WHERE 1=1
		      $conditions
SQL;
        $param = array( 'NRPRODUTO_ID' => $nrProdutoId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
        }
        return $registers;
    }

    public function findTipoMedidaServ($conditions, $nrOrg, $limit, $offset) {
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT CDTPMEDIDASERV, DSTPMEDIDASERV, IDTPCOLETAMEDIDASERV, IDATIVO, DSTPMEDIDASERV AS DSUSUTPMEDIDASERV, CDTPMEDIDASERV AS CDUSUTPMEDIDASERV
                   FROM FCT_TIPOMEDIDASERV
                  WHERE NRORG = :NRORG )
		 WHERE 1 = 1
		       $conditions
      ORDER BY DSTPMEDIDASERV
SQL;
        $param = array( 'NRORG' => $nrOrg );
        $query = $this->getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findVariableMeasureByExternalProductId($productId, $cdtpmedidaserv = null, $cdservorca = '', $nmservorca = '')
    {
        $cdtpmedidaservFilter = '';
        if($cdtpmedidaserv) {
            $cdtpmedidaservFilter = "AND CMV.CDTPMEDIDASERV = $cdtpmedidaserv";
        }
        $query = <<<SQL
		SELECT CMV.NRPRODUTO_ID_REL, CMV.ID, CMV.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV, CMV.PHP_SCRIPT_COUNT,
               CMV.PHP_SCRIPT_LOG, CMV.QTD_DIAS_AVISO, CMV.QTD_DIAS_REAPLICAR,
               CMV.NRPRODUTO_ID, PRO.NMPRODUTO, PRO.NREXTERNAL_PRODUCT_ID,
               '$cdservorca' AS CDSERVORCA, '$nmservorca' AS NMSERVORCA
          FROM OM_CONFIG_MEDIDA_VARIAVEL CMV
          JOIN OM_PRODUTO PRO ON PRO.ID = CMV.NRPRODUTO_ID
          JOIN FCT_TIPOMEDIDASERV TMS ON TMS.CDTPMEDIDASERV = CMV.CDTPMEDIDASERV
         WHERE PRO.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
           AND CMV.IDATIVO = 'S'
           $cdtpmedidaservFilter
           AND (CMV.NRPRODUTO_ID_REL = PRO.ID OR CMV.NRPRODUTO_ID_REL IS NULL)
           OR (CMV.NRPRODUTO_ID_REL = (SELECT ID 
                                        FROM OM_PRODUTO OMP
                                       WHERE OMP.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID)
                                         AND CMV.IDATIVO = 'S'
                                         $cdtpmedidaservFilter)
SQL;
        $param = array( 'PRODUCT_ID' => $productId );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findVariableMeasureRelatedOrg($cdtpmedidaserv, $cdservorca, $productId, $productName)
    {
        $query = <<<SQL
		    SELECT CMV.ID, CMV.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV, CMV.PHP_SCRIPT_COUNT,
                   CMV.PHP_SCRIPT_LOG, CMV.QTD_DIAS_AVISO, CMV.QTD_DIAS_REAPLICAR,
                   $productId AS NREXTERNAL_PRODUCT_ID, $productId AS  NRPRODUTO_ID, '$productName' AS NMPRODUTO,
                   SEO.CDSERVORCA, SEO.NMSERVORCA
              FROM OM_COLETA_MEDIDA_VARIAVEL CMV
              JOIN FCT_TIPOMEDIDASERV TMS ON TMS.CDTPMEDIDASERV = CMV.CDTPMEDIDASERV
              JOIN FCT_SERVICOORCA SEO ON SEO.CDSERVORCA = CMV.CDSERVORCA
             WHERE CMV.CDTPMEDIDASERV = :CDTPMEDIDASERV
               AND CMV.CDSERVORCA = :CDSERVORCA
               AND CMV.IDATIVO = 'S'
SQL;
        $param = array( 'CDTPMEDIDASERV' => $cdtpmedidaserv, 'CDSERVORCA' => $cdservorca);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findConfigCollect($productId, $organizationId)
    {
        $query = <<<SQL
		    SELECT OCC.ID, OCC.QTD_DIAS_REAPLICAR, TO_CHAR(OCC.DT_LIMITE, 'DD/MM/YYYY') AS DT_LIMITE, OCC.PHP_SCRIPT_LOG,
                   OMP.ID AS NRPRODUTO, OMP.NMPRODUTO, OMP.NREXTERNAL_PRODUCT_ID
              FROM OM_CONFIG_COLETA OCC
              JOIN OM_PRODUTO OMP ON OMP.ID = OCC.NRPRODUTO_ID
        INNER JOIN OM_CONFIG_COLETA_ORG OCO ON OCO.NRCONFCOL_ID = OCC.ID AND OCO.NRORG_ID = :NRORG_ID              
             WHERE OMP.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
               AND OCC.IDATIVO = 'S'
               AND TRUNC(SYSDATE) <= TRUNC(OCC.DT_LIMITE) 
SQL;
        $param = array( 'PRODUCT_ID' => $productId, 'NRORG_ID' => $organizationId);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findServerById($serverId)
    {
        $query = <<<SQL
            SELECT *
              FROM SERVER S
             WHERE S.ID = :SERVER_ID
SQL;
        $param = array( 'SERVER_ID' => $serverId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findActualCustomerContracts($collectType = \Teknisa\Model\Teknisa\Entities\OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_TEKNISA_TYPE,
                                                $productId = null, $organizationId = null, $variableMeasureId = null) {
        $productFilter = $variableMeasureFilter = $organizationFilter = '';
        if($productId) {
            $productFilter = "AND (PRO.NREXTERNAL_PRODUCT_ID = $productId)";
        }
        if($variableMeasureId) {
            $variableMeasureFilter = "AND REL.CDTPMEDIDAVARIAVEL = '$variableMeasureId'";
        }
        if($organizationId) {
            $organizationFilter = "AND ORG.NRORG = $organizationId";
        }
        $query = <<<SQL
		SELECT CONT.DSREFERENCIACONT AS DSREFERENCIAORCA, VENC.NRSEQCONTRATO,
               VENC.NRMEDICAOFATURA AS NRVENCPREVCONT,
               ORG.NMRAZSOCIALORG AS NMRAZSOCCLIE, ORG.NMORGANIZACAO AS NMFANTCLIE, ORG.NRORG,
               TO_CHAR(VENC.DTINICOMPET, 'DD/MM/YYYY') AS DTINICOMPET,
               TO_CHAR(VENC.DTFINCOMPET, 'DD/MM/YYYY') AS DTFINCOMPET,
               TMS.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV, SEO.CDSERVORCA, SEO.NMSERVORCA,
               NVL(PRO.NREXTERNAL_PRODUCT_ID, 0) AS NREXTERNAL_PRODUCT_ID, PRO.NMPRODUTO
          FROM FCT_MEDICAOFATURA VENC
    INNER JOIN FCT_CONTRATO CONT
            ON VENC.NRSEQCONTRATO = CONT.NRSEQCONTRATO
    INNER JOIN ORGANIZACAO ORG
            ON CONT.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
               $organizationFilter
    INNER JOIN FCT_RELACMEDIDACONTRATO REL
            ON REL.NRSEQCONTRATO = CONT.NRSEQCONTRATO
    LEFT JOIN FCT_SERVICOORCA SEO
            ON SEO.CDSERVORCA = REL.CDSERVORCA
     LEFT JOIN OM_PRODUTO PRO
            ON TO_CHAR(PRO.ID) = REPLACE(REL.IDELEMENTORAIZ, 'P_', '')
    INNER JOIN FCT_TIPOMEDIDASERV TMS
            ON TMS.CDTPMEDIDASERV = REL.CDTPMEDIDAVARIAVEL
     LEFT JOIN FCT_FATURACONT FNT
            ON FNT.NRMEDICAOFATURA = VENC.NRMEDICAOFATURA
         WHERE CONT.IDSITUACAOCONTRATO = 'A'
           AND VENC.IDSITUACAO NOT IN (4, 5)
           AND TRUNC(SYSDATE) BETWEEN TRUNC(VENC.DTINICOMPET) AND TRUNC(VENC.DTFINCOMPET)
           AND TMS.IDTPCOLETAMEDIDASERV = :IDTPCOLETAMEDIDASERV
           AND FNT.NRMEDICAOFATURA IS NULL
           AND (REL.IDELEMENTORAIZ IS NULL OR REL.IDELEMENTORAIZ IN (SELECT IDELEMENTO
                                                                       FROM FCT_ITEMCONTRATO
                                                                      WHERE NRSEQCONTRATO = VENC.NRSEQCONTRATO))
               $productFilter
               $variableMeasureFilter
      ORDER BY ORG.NMORGANIZACAO, PRO.NMPRODUTO
SQL;
        $param = array (
            'IDTPCOLETAMEDIDASERV' => $collectType
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findPendentContractsSendVariableMeasure() {
        $query = <<<SQL
		SELECT ORG.NRORG, ORG.NMORGANIZACAO, 
		       PRO.NMPRODUTO, PRO.ID AS NRPRODUTO_ID,
		       TO_CHAR(VENC.DTINICOMPET, 'DD/MM/YYYY') AS DTINICOMPET,
               TO_CHAR(VENC.DTFINCOMPET, 'DD/MM/YYYY') AS DTFINCOMPET,
		       VENC.NRSEQCONTRATO
          FROM FCT_MEDICAOFATURA VENC
    INNER JOIN FCT_CONTRATO CONT
            ON VENC.NRSEQCONTRATO = CONT.NRSEQCONTRATO
           AND VENC.NRORG         = CONT.NRORG
    INNER JOIN ORGANIZACAO ORG
            ON CONT.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
    INNER JOIN FCT_RELACMEDIDACONTRATO REC
            ON REC.NRSEQCONTRATO = CONT.NRSEQCONTRATO
           AND REC.NRORG          = CONT.NRORG
    INNER JOIN OM_PRODUTO PRO
            ON TO_CHAR(PRO.ID) = REPLACE(REC.IDELEMENTORAIZ, 'P_', '')
         WHERE CONT.IDSITUACAOCONTRATO = 'A'
           AND VENC.IDSITUACAO NOT IN (4,5)
           AND TRUNC(SYSDATE) BETWEEN TRUNC(VENC.DTINICOMPET) AND TRUNC(VENC.DTFINCOMPET)
      ORDER BY ORG.NMORGANIZACAO, PRO.NMPRODUTO
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    private function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $max = $offset + $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }

        return $query;
    }
}